# epnode6.js
![alt text](epnode6.png "epnode6.js")

This is a helper for ePages 6 written in JavaScript for node.js. It provides the functionality to
pack the JavaScript, Less and CSS files of a store as well as creating a minified version of the entire
*~/lib* directory.
It also searches for missing test cases and reports the current health of our test environment.


## Installation

You can install this node module via `npm`. Because this module is not available in public
repositories, you will need a tarball containing the module. Locate this tarball, change into the
containing directory and issue the following command:

    npm install -g node-epages6.tar.gz

After installation make sure to run `npm install` from within the node-epages6 directory.

## Synopsis
### Command Line Iterface (CLI)
>#### Usage:
    $ epnode6.js [-h] [-v] [-l] {minify,pack,builddoc,testcases} ...
>#### Description:
    ePages 6 node.js helper
>
        Optional arguments:
          -h, --help            Show this help message and exit.
          -v, --version         Show program's version number and exit.
          -l, --extended-log    Activate extended logging and stack tracing
>
    subcommands:
      {minify,minify-lib,pack-lib,build,builddoc,testcases}

### Subcommands (modules)
The CLI is divided into six subcommands which can be accessed separately.
#### Module: pack-lib
>##### Usage:
    $ epnode6.js pack-lib [-h] -s STORE_TYPE [-v VERSION] [-w WEBROOT] [-t {less,js}] [-p {bo,sf,mobilesf,sf_appointment,pageeditor,editor,testsuite}]
>##### Description:
    Create Packages for JavaScript and CSS files from the specified profile.
>
    Optional arguments:
      -h, --help    
          Show this help message and exit.
      -s STORE_TYPE, --store-type STORE_TYPE
          Store type e.g. Store
      -v STORE_VERSION, --store-version STORE_VERSION
          Store version e.g. 6.17.0
      -w WEBROOT, --webroot WEBROOT
          Epages webroot defaults to $EPAGES_WEBROOT
      -t STORE_TYPE, --store-type STORE_TYPE
          Store type e.g. Store
      -t {less,js}, --package-type {less,js}
          Type of package that should be built
      -p {bo,sf,mobilesf,sf_appointment,pageeditor,editor,testsuite}, --profile {bo,sf,mobilesf,sf_appointment,pageeditor,editor,testsuite}
          Profile name of resulting package

>##### Example:
    $ epnode6.js pack --js --css -p sf -p mobilesf -p editor -p testsuite -t Store -v 6.17.0

####Module: minify-lib
>##### Usage:
    epnode6 minify-lib [-h] -s STORE_TYPE [-v VERSION] [-w WEBROOT] [-f FILTER]
>##### Description:
    Minify all JavaScript and CSS files within the lib folder.
>
    Optional arguments:
      -h, --help    
          Show this help message and exit.
      -s STORE_TYPE, --store-type STORE_TYPE
          Store type e.g. Store
      -v VERSION, --version VERSION
          Store version e.g. 6.17.0
      -w WEBROOT, --webroot WEBROOT
          Epages webroot defaults to $EPAGES_WEBROOT
      -f FILTER, --filter FILTER
          Filter files to be minified e.g. /**/*.js
>#####Example:
>
    epnode minify -t Store -v 6.17.0 -f /**/*.ca.json

####Module: minify
>##### Usage:
    epnode6 minify [-h] -s SOURCE -t TARGET [-f FILTER]
>##### Description:
    Minify the specified source file or folder to the specified target.
>
    Optional arguments:
      -h, --help            
          Show this help message and exit.
      -s SOURCE, --source SOURCE
          Source that should be minified. Can either be a file or a folder.
      -t TARGET, --target TARGET
          The target for the minfied file or folder. If <source> is a folder, <target> must be a folder too.
      -f FILTER, --filter FILTER
          Filter files to be minified e.g. /**/\*.ca.json
>#####Example:
>
    $ epnode6.js minify -s lib -t lib-min

####Module: testcases
>#####Usage:
    $ epnode6.js testcases [-h] [-c CARTRIDGE]
>##### Description:
    Check for the presence of testcases
>
    Optional arguments:
      -h, --help    
          Show this help message and exit.
      -c CARTRIDGE, --cartridge CARTRIDGE
          Check if the cartridge has testcases [multi]

>#### Example:
    $ epnode6.js testcases -c jquery

####Module: builddoc
>##### Usage:
    $ epnode6.js builddoc [-h] -i INCLUDE [-o OUTPUT] [-c CARTRIDGE_FOLDER]
>##### Description
    Create the jsduck documentation.
>
    Optional arguments:
      -h, --help    
          Show this help message and exit.
      -i INCLUDE, --include INCLUDE
          A JSON file which enumerates the files to include in the documentation.
      -o OUTPUT, --output OUTPUT
          The output folder for the documentation
      -c CARTRIDGE_FOLDER, --cartridge-folder CARTRIDGE_FOLDER
          The folder containing the cartridges, typically /srv/epages/eproot/Cartridges

>#####Example:
    $ epnode6.js builddoc -i sources.json -o ./