use strict;

use Getopt::Long;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw( BusinessUnits );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetStoreTypeCartridges );
use JSON;

use constant SUBS => {'GetStoreTypeCartridges' => \&_GetStoreTypeCartridges, 'GetEnv' => \&_GetEnv, 'GetVersion' => \&_GetVersion};

sub Main {

    my ($Help, $Sub, $Param);

    GetOptions(
        'help'  => \$Help,
        'sub=s' => \$Sub,
        'param=s' => \$Param,
    );

    usage() if $Help;
    usage('Undefined parameter sub.') unless defined $Sub;
    usage("Unknown subroutine $Sub.") unless defined SUBS->{"$Sub"};

    eval { print _TO_JSON(SUBS->{"$Sub"}->($Param)); };
    print to_json(GetError()) if ExistsError();

    return;
}

sub _TO_JSON {
    my ($Variable) = @_;

    if (ref($Variable) eq 'HASH' or ref($Variable) eq 'ARRAY') {
        return to_json($Variable);
    }

    $Variable =~ s/(\\|")/\\$1/g;

    return '"'.$Variable.'"';
}

sub _GetVersion {
    return BusinessUnits()->{Site}{Cartridges}{'DE_EPAGES::Core'};
}

sub _GetStoreTypeCartridges {
    my ($StoreType) = @_;

    return GetStoreTypeCartridges($StoreType);
}

sub _GetEnv {
    my ($Param) = @_;
    return unless defined $Param;

    $Param = uc($Param);

    return $ENV{"EPAGES_$Param"};
}

sub usage {
    my ($Msg) = @_;
    print "$Msg\n\n" if $Msg;
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    Perl interface for nodejs modules for e.g. build process and packaging.

Options:
    -sub  name of sub (required)
    -param  sub parameter (optional)

Flags:
    -help           show the command line options           (optional)

Examples:
    perl $0 -sub GetCartridges
    perl $0 -sub GetEnv -param Webroot

END_USAGE
exit 1;
}

RunScript(
    'Sub' => \&Main
);