/*jslint nomen:true,regexp:true,node:true,unparam:true*/
(function () {

    'use strict';

    var winston = require('winston'),
        _ = require('underscore'),
        logger = new winston.Logger();

    var log = function (module, level, message, metadata) {
        logger.log(level, module + ": " + message, metadata || {});
    };

    module.exports = {
        levels : ['error', 'warn', 'info', 'verbose', 'debug', 'silly'],

        // transport is 'File', 'Console'
        add: function (transport, options) {
            logger.add(winston.transports[transport], options);
        },

        getLogger: function (options) {
            return {
                silly: function (message, metadata) {
                    log(options.module, 'silly', message, metadata);
                },
                debug: function (message, metadata) {
                    log(options.module, 'debug', message, metadata);
                },
                verbose: function (message, metadata) {
                    log(options.module, 'verbose', message, metadata);
                },
                info: function (message, metadata) {
                    log(options.module, 'info', message, metadata);
                },
                warn: function (message, metadata) {
                    log(options.module, 'warn', message, metadata);
                },
                error: function (message, metadata) {
                    log(options.module, 'error', message, metadata);
                }
            };
        },

        getLogLevel: function (args) {
            var logLevel = 0;

            // get log level
            _.each(args, function (argument, ix) {
                // set log level if log-level argument was found
                if (argument === "-l") {
                    logLevel = args[ix + 1];
                }
            });

            return logLevel;
        },
    };
}());