/*jslint indent:4,node:true,nomen:true*/
/*global describe,before,it,after*/
(function () {

    'use strict';

    var expect = require('expect.js'),
        fs     = require('fs'),
        sinon  = require('sinon'),
        child_process = require('child_process'),
        path = require('path');

    describe('Running builddoc.js', function () {
        var builddocLocation = path.normalize(path.join(__dirname, '../lib/', 'builddoc')),
            builddoc,
            lastCmd;

        before(function () {
            // remove perl and child_process module from cache
            require.cache[builddocLocation + '.js'] = undefined;
            require.cache.child_process = undefined;

            // mock child_process.exec
            sinon.stub(child_process, 'exec', function (cmd, options, callback) {
                if (!callback) {
                    callback = options;
                    options = undefined;
                }

                lastCmd = cmd;
                callback();
            });

            builddoc = require(builddocLocation);
        });

        it('should build the documentation', function (callback) {
            builddoc({
                cartridges: '/the/cartridges',
                input: path.join(__dirname, 'fs/doc/includes.json'),
                output: path.join(__dirname, 'fs/doc/output')
            }, function (error) {
                expect(error).to.not.be.ok();

                expect(lastCmd).to.be('jsduck "' + path.join('/the', 'cartridges', 'DE_EPAGES', 'a.js') + '" "' + path.join('/the', 'cartridges', 'DE_EPAGES', 'b.js') + '" "' + path.join('/the', 'cartridges', 'DE_EPAGES', 'c.js') + '" "' + path.join('/the', 'cartridges', 'DE_EPAGES', 'd.js') + '" "' + path.join('/the', 'cartridges', 'DE_EPAGES', 'e.js') + '"  -o "' + path.join(__dirname, 'fs/doc/output') + '"');

                callback();
            });
        });

        after(function (callback) {
            // restore child process module
            child_process.exec.restore();

            // remove output folder
            fs.rmdir(path.join(__dirname, 'fs/doc/output'), callback);
        });

    });

}());
