/*jslint node: true*/
/*globals describe, it*/

(function () {
    'use strict';

    var expect = require('expect.js'),
        //sinon  = require('sinon'),
        cfg = require('../lib/cfg');

    describe('Checking cfg.js', function () {

        describe('pack-lib module', function () {
            it('should exist', function () {
                expect(cfg['pack-lib']).to.be.an(Object);
            });

            it('should return correct description', function () {
                expect(cfg['pack-lib'].description).to.be('Create Packages for JavaScript and CSS files from the specified profile.');
            });

            it('should return correct arguments', function () {
                expect(cfg['pack-lib'].args).to.be.an(Array);
                expect(cfg['pack-lib'].args).to.eql([{
                    alias: ['-s', '--store-type'],
                    nargs: 1,
                    required: true,
                    help: 'Store type e.g. Store'
                }, {
                    alias: ['-v', '--version'],
                    nargs: 1,
                    required: false,
                    help: 'Store version e.g. 6.17.0'
                }, {
                    alias: ['-w', '--webroot'],
                    nargs: 1,
                    required: false,
                    help: 'Epages webroot defaults to $EPAGES_WEBROOT'
                }, {
                    alias: ['-t', '--package-type'],
                    nargs: 1,
                    required: false,
                    choices: ['less', 'js'],
                    help: 'Type of package that should be built'
                },
                {
                    alias: ['-p', '--profile'],
                    action: 'append',
                    nargs: 1,
                    required: false,
                    choices: ['bo', 'sf', 'mobilesf', 'sf_appointment', 'pageeditor', 'editor', 'testsuite'],
                    help: 'Profile name of resulting package'
                }]);
            });

            it('should return correct usage example', function () {
                expect(cfg['pack-lib'].example).to.be('Example: epnode pack --js --css -p sf -p mobilesf -p editor -p testsuite -t Store -v 6.17.0');
            });

            it('should have an executable function calling the actual module', function () {
                expect(cfg['pack-lib'].exec).to.be.a(Function);
            });
        });

        describe('minify-lib module', function () {
            it('should exist', function () {
                expect(cfg['minify-lib']).to.be.an(Object);
            });

            it('should return correct description', function () {
                expect(cfg['minify-lib'].description).to.be('Minify all JavaScript and CSS files within the lib folder.');
            });

            it('should return correct arguments', function () {
                expect(cfg['minify-lib'].args).to.be.an(Array);
                expect(cfg['minify-lib'].args).to.eql([{

                    alias: ['-s', '--store-type'],
                    nargs: 1,
                    required: true,
                    help: 'Store type e.g. Store'
                }, {

                    alias: ['-v', '--version'],
                    nargs: 1,
                    required: false,
                    help: 'Store version e.g. 6.17.0'
                }, {

                    alias: ['-w', '--webroot'],
                    nargs: 1,
                    required: false,
                    help: 'Epages webroot defaults to $EPAGES_WEBROOT'
                }, {

                    alias: ['-f', '--filter'],
                    nargs: 1,
                    required: false,
                    help: 'Filter files to be minified e.g. /**/*.js'
                }]);
            });

            it('should return correct usage example', function () {
                expect(cfg['minify-lib'].example).to.be('Example: epnode minify -t Store -v 6.17.0 -f /**/*.ca.json');
            });

            it('should have an executable function calling the actual module', function () {
                expect(cfg['minify-lib'].exec).to.be.a(Function);
            });
        });

        describe('minify module', function () {
            it('should exist', function () {
                expect(cfg.minify).to.be.an(Object);
            });

            it('should return correct description', function () {
                expect(cfg.minify.description).to.be('Minify the specified source file or folder to the specified target.');
            });

            it('should return correct arguments', function () {
                expect(cfg.minify.args).to.be.an(Array);
                expect(cfg.minify.args).to.eql([{
                    alias: ['-s', '--source'],
                    nargs: 1,
                    required: true,
                    help: 'Source that should be minified. Can either be a file or a folder.'
                }, {
                    alias: ['-t', '--target'],
                    nargs: 1,
                    required: true,
                    help: 'The target for the minfied file or folder. If <source> is a folder, target must be a folder too.'
                }, {
                    alias: ['-f', '--filter'],
                    nargs: 1,
                    required: false,
                    help: 'Filter files to be minified e.g. /**/*.js'
                }]);
            });

            it('should return correct usage example', function () {
                expect(cfg.minify.example).to.be('Example: epnode minify -s lib -t lib-min');
            });

            it('should have an executable function calling the actual module', function () {
                expect(cfg.minify.exec).to.be.a(Function);
            });
        });

        describe('build-ui module', function () {
            it('should exist', function () {
                expect(cfg['build-ui']).to.be.an(Object);
            });

            it('should return correct description', function () {
                expect(cfg['build-ui'].description).to.be('Create Packages for JavaScript and CSS files for the specified store type.');
            });

            it('should return correct arguments', function () {
                expect(cfg['build-ui'].args).to.be.an(Array);
                expect(cfg['build-ui'].args).to.eql([{
                    alias: ['-s', '--store-type'],
                    nargs: 1,
                    required: true,
                    help: 'Store type e.g. Store'
                }, {
                    alias: ['-v', '--version'],
                    nargs: 1,
                    required: false,
                    help: 'Store version e.g. 6.17.0'
                }, {
                    alias: ['-w', '--webroot'],
                    nargs: 1,
                    required: false,
                    help: 'Epages webroot defaults to $EPAGES_WEBROOT'
                }, {
                    alias: ['-f', '--filter'],
                    nargs: 1,
                    required: false,
                    help: 'Filter files to be minified e.g. /**/*.js'
                }]);
            });

            it('should return correct usage example', function () {
                expect(cfg['build-ui'].example).to.be('Example: epnode build-ui -s Store');
            });

            it('should have an executable function calling the actual module', function () {
                expect(cfg['build-ui'].exec).to.be.a(Function);
            });
        });

        describe('testcases module', function () {
            it('should exist', function () {
                expect(cfg.testcases).to.be.an(Object);
            });

            it('should return correct description', function () {
                expect(cfg.testcases.description).to.be('Check for the presence of testcases.');
            });

            it('should return correct arguments', function () {
                expect(cfg.testcases.args).to.be.an(Array);
                expect(cfg.testcases.args).to.eql([{
                    alias: ['-c', '--cartridge'],
                    nargs: 1,
                    required: false,
                    action: 'append',
                    help: 'Check if the cartridge has testcases [multi]',
                    example: 'Example: epnode testcases -c jquery -c presentation'
                }]);
            });

            it('should return correct usage example', function () {
                expect(cfg.testcases.example).to.be('Example: epnode testcases -c jquery');
            });

            it('should have an executable function calling the actual module', function () {
                expect(cfg.testcases.exec).to.be.a(Function);
            });
        });

        describe('builddoc module', function () {
            it('should exist', function () {
                expect(cfg.builddoc).to.be.an(Object);
            });

            it('should return correct description', function () {
                expect(cfg.builddoc.description).to.be('Create the jsduck documentation.');
            });

            it('should return correct arguments', function () {
                expect(cfg.builddoc.args).to.be.an(Array);
                expect(cfg.builddoc.args).to.eql([{
                    alias: ['-i', '--include'],
                    nargs: 1,
                    required: true,
                    help: 'A JSON file which enumerates the files to include in the documentation.'
                }, {
                    alias: ['-o', '--output'],
                    nargs: 1,
                    required: false,
                    help: 'The output folder for the documentation'
                }, {
                    alias: ['-c', '--cartridges'],
                    nargs: 1,
                    required: false,
                    help: 'The folder containing the cartridges, typically /srv/epages/eproot/Cartridges'
                }]);
            });

            it('should return correct usage example', function () {
                expect(cfg.builddoc.example).to.be('Example: epnode builddoc -i sources.json -o ./');
            });

            it('should have an executable function calling the actual module', function () {
                expect(cfg.builddoc.exec).to.be.a(Function);
            });
        });
    });
}());